/*
 * Last updated: 2011/12/06
 * ~Philip Peng
 * 
 * This file is part of BomberNome.
 * 
 * BomberNome is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * BomberNome is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with BomberNome.	If not, see <http://www.gnu.org/licenses/>.
 */

#include "bombernome.h"

// Called by main, init hardware
void setup_input() {
	
	// Input ports
	// PTM[1:0] are used for hardwired for other stuff, so we
	// can only use PTM[5:2]
	DDRM &= 0xC3;	// PTM[5:2] is for the input from FireFly
	
	// RTICTL = [---][RTR6][RTR5][RTR4][RTR3][RTR2][RTR1][RTR0]
	// RTR[6:4] = Real-Time Interrupt Rescale Rate
	// RTR[3:0] = Real Time Interrupt Modulus Counter
	RTICTL = INPUT_PRESCALER;	// Set RTI timeout period
	
	// CRGINT = [RTIE][---][---][LOCKIE][---][---][SCMIE][---]
	// RTIE = Real Time Interrupt Enable (want 1)
	// LOCKIE = Lock Interrupt Enable (want 0?)
	// SCMIE = Self-Clock Mode Interrupt Enable (want 0?)
	CRGINT = 0x80;			// Enable RTI interrupt
}

// Called by input_interrupt, update player actions
void update_actions() {
	byte input;
	player_t *player;
	
	// We can only use PTM[5:2]
	// 0b00111100 = 0x3C
	input = PTM & 0x3C;
	if (input & 0x04) {		// 0b--xxx1--
		player = player_2;
	} else {				// 0b--xxx0--
		player = player_1;
	}
	// Check for death
	if (player->action == ACTION_DEAD ||
		player->action == ACTION_DYING) {
		return;
	}
	
	switch (input & 0x38) {	// 0b--111x--
		case 0x00:			// 0b--000x--
			// No assigned action, fall through
		case 0x08:			// 0b--001x--
			// No assigned action, fall through
		case 0x10:			// 0b--010x--
			player->action = ACTION_NONE;
			break;
		case 0x18:			// 0b--011x--
			player->action = ACTION_BOMB;
			break;
		case 0x20:			// 0b--100x--
			player->action = ACTION_UP;
			break;
		case 0x28:			// 0b--101x--
			player->action = ACTION_DOWN;
			break;
		case 0x30:			// 0b--110x--
			player->action = ACTION_LEFT;
			break;
		case 0x38:			// 0b--111x--
			player->action = ACTION_RIGHT;
			break;
	}
}

#ifdef TEST_INPUT
// For testing without real input
void simulate_input() {
  static unsigned short i;
  i++;                  
	if (i == 15) i = 0;
	if (i < 3) {
		player_1->action = ACTION_RIGHT;
		player_2->action = ACTION_DOWN;
		player_3->action = ACTION_UP;
		player_4->action = ACTION_LEFT;
	} else if (i < 6) {
		player_1->action = ACTION_NONE;
		player_2->action = ACTION_NONE;
		player_3->action = ACTION_NONE;
		player_4->action = ACTION_NONE;
	} else if (i < 9) {
		player_1->action = ACTION_LEFT;
		player_2->action = ACTION_UP;
		player_3->action = ACTION_DOWN;
		player_4->action = ACTION_RIGHT;
	} else if (i < 12) {
		player_1->action = ACTION_NONE;
		player_2->action = ACTION_NONE;
		player_3->action = ACTION_NONE;
		player_4->action = ACTION_NONE;
	}  
}
#endif
